/*

INVLVL -- Deletes all users with invalid access levels

Version 1.4  (12/16/89)
Written by Bob Quinlan of Austin, TX, USA

Copyright 1989 by Bob Quinlan

Compatible with GT version 15


This program goes through the GT user file and deletes all user records
which do not have a legal access level.  It prints out the user name of
every record it deletes (with spaces substituted for unprintables).  If
any errors occur it will return ERRORLEVEL 1.  Normally it returns
ERRORLEVEL 0.

NOTICE:  You may use, copy, and distribute this program freely as long
as you insure that both the executable and the documentation (.DOC)
files are included in the distribution package.  The source code does
not need to be included.  You may modify this program and document, so
long as reasonable credit is given to the original author if a
substantial portion of the original remains intact.  The author is not
responsible for any losses which may occur either directly or indirectly
as a result of using this program.

HISTORY:
Version 1.4  (12/16/89) -- Fixed bugs.
Version 1.3  (12/15/89) -- Cleaned up file locking code.
Version 1.2  (12/13/89) -- Added support for multiple node systems.
Version 1.1   (8/15/89) -- Added file locking to support multiple node
			   systems.  Added name output.
Version 1.0   (7/19/89) -- Original release.  Written in Turbo C.

*/

/*  Tiny memory model  */

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <ctype.h>
#include <string.h>
#include "dram.h"	/*  From the GT Developer's Toolkit  */

#define MAXLINE		128

#define MAXFLEN		0x7FFFFFFFL

int  set_share(void);
void flock(FILE *);
void funlock(FILE *);

int  share;


/************/
/*  INVLVL  */
/************/

main(void)
{
USER_RECORD user;

char   line[MAXLINE];
char   filename[MAXLINE];

char   gtpath[MAXLINE];
char   lanpath[MAXLINE];

FILE   *gtcnf_fp;
FILE   *user_fp;

fpos_t oldpos;
fpos_t newpos;

int    len;

int    i;


printf("INVLVL 1.4 -- Copyright 1989 by Bob Quinlan (12/16/89)\n");

/*  Initialize share  */
set_share();

/*  Get the GTPATH value from the environment  */
strcpy(gtpath, getenv("GTPATH"));
/*  Add final backslash to path if not already present  */
i = strlen(gtpath);
if (--i >= 0)
    if ((gtpath[i] != '\\') && (gtpath[i] != ':'))
	strcat(gtpath, "\\");

/*  Append GT.CNF to the GTPATH  */
strcpy(filename, gtpath);
strcat(filename, "GT.CNF");
/*  Open GT.CNF  */
if ((gtcnf_fp = fopen(filename, "rt")) == NULL)
    {
    fprintf(stderr, "INVLVL: Unable to open %s\n", filename);
    exit(1);
    }
/*  Assign the default gtpath as the default lan path  */
strcpy(lanpath, gtpath);
/*  Read GT.CNF to get the lan path  */
while (fgets(line, MAXLINE, gtcnf_fp) != NULL)
    if (strnicmp(line, "lr=", 3) == 0)
        {
        /*  Strip any trailing whitespace or line terminators  */
        i = strcspn(line, " \r\n");
        line[i] = '\0';
        if (strlen(line) > 3)
            {
            /*  Assign the lan path  */
            strcpy(lanpath, line+3);
            /*  Add final backslash to path if not already present  */
            i = strlen(lanpath);
            if (--i >= 0)
                if ((lanpath[i] != '\\') && (lanpath[i] != ':'))
                    strcat(lanpath, "\\");
            }
        break;
        }
/*  Close GT.CNF  */
fclose(gtcnf_fp);

/*  Append USER.CTL to the lan path  */
strcpy(filename, lanpath);
strcat(filename, "USER.CTL");
/*  Open the user file  */
if ((user_fp = fopen(filename, "r+b")) == NULL)
    {
    fprintf(stderr, "INVLVL: Unable to open %s\n", filename);
    exit(1);
    }
/*  Lock the user file to prevent shared access  */
flock(user_fp);

/*  Skip the control record  */
fread(&user, sizeof(USER_RECORD), 1, user_fp);
fgetpos(user_fp, &oldpos);
/*  Scan for undeleted users with invalid access levels  */
while (fread(&user, sizeof(USER_RECORD), 1, user_fp) > 0)
    {
    fgetpos(user_fp, &newpos);
    if ((user.user_deleted == 0) && (!isalnum(user.user_privel)))
	{
	/*  Filter and print the user name to be deleted  */
	len = strlen((char *)user.user_name);
	for (i=0; i<len; i++)
	    if ((user.user_name[i] < 32) || (user.user_name[i] > 126))
		user.user_name[i] = '#';
	puts((char *)user.user_name);
	/*  Mark the record as deleted  */
	user.user_deleted = 1;
	fsetpos(user_fp, &oldpos);
	fwrite(&user, sizeof(USER_RECORD), 1, user_fp);
	fsetpos(user_fp, &newpos);
	}
    oldpos = newpos;
    }

/*  Unlock and close the user file  */
funlock(user_fp);
fclose(user_fp);

return 0;
}


/*  Determine whether SHARE is loaded  */
int set_share(void)
{
FILE *tmp_fp;

tmp_fp = tmpfile();
if (lock(fileno(tmp_fp), 0L, MAXFLEN) == 0)
    {
    unlock(fileno(tmp_fp), 0L, MAXFLEN);
    share = 1;
    }
else
    share = 0;
fclose(tmp_fp);
return share;
}


/*  Wait until the stream can be locked if SHARE is loaded  */
void flock(FILE *file)
{
if (share)
    while (lock(fileno(file), 0L, MAXFLEN) != 0);
}


/*  Unlock the stream if SHARE is loaded  */
void funlock(FILE *file)
{
if (share)
    unlock(fileno(file), 0L, MAXFLEN);
}
